/**
  ******************************************************************************
  * @file    fm33a0xxev_uart.h
  * @author  FM33A0XXEV Application Team
  * @version V1.0.0
  * @date    16-April-2020
  * @brief   This file contains all the functions prototypes for the UART firmware library.  
  ******************************************************************************
  */
  
/* Define to prevent recursive inclusion -------------------------------------*/

#ifndef __FM33A0XXEV_UART_H
#define __FM33A0XXEV_UART_H

#ifdef __cplusplus
 extern "C" {
#endif
   
/* Includes ------------------------------------------------------------------*/

#include "FM33A0XXEV.h" 
#include "fm33a0xxev_CMU.h" 
   
typedef enum
{
	RxInt,
	TxInt	
}UART_IntTypeDef;

typedef enum
{
	Seven7Bit,		
	Eight8Bit,
	Nine9Bit,
	Six6Bit	
}UART_DataBitTypeDef;

typedef enum
{
	NONE,
	EVEN,
	ODD	
}UART_ParityBitTypeDef;

typedef enum
{
	OneBit,
	TwoBit
}UART_StopBitTypeDef;

typedef struct
{
	uint32_t				      BaudRate;		//
	UART_DataBitTypeDef		DataBit;		//λ
	UART_ParityBitTypeDef	ParityBit; 	//Уλ
	UART_StopBitTypeDef		StopBit;		//ֹͣλ
	uint32_t 				      ClockSrc;   // ʱԴѡ
}UART_SInitTypeDef;	

typedef struct
{
	FunState				RXBF_IE;		    //ж
	FunState				TXBE_IE;		//ͻж
	FunState				TXSE_IE;		//ͻҷͼĴж
	uint32_t				SPBRG;		  //ʲĴ
	uint32_t				PDSEL;		  //ݳѡ
	uint32_t				PARITY;		  //Уλ
  FunState				RXTO_IE;		//ճʱж
	FunState				ERRIE;		  //жʹܿ
	FunState				RXEN;		    //ģʹܿ
	uint32_t				STOPSEL;	  //ֹͣλѡ
	FunState				TXEN;		    //ģʹܿ
	FunState				IREN;		    //ͺʹλ
	FunState				RXDFLAG;	  //ȡλ
	FunState				TXDFLAG;	  //ȡλ	
}UART_InitTypeDef;	

#define	UARTIR_CR_IRFLAG_Pos	15	/* ƺƷʱĬ (Infra Red Flag)
0
1 */
#define	UARTIR_CR_IRFLAG_Msk	(0x1U << UARTIR_CR_IRFLAG_Pos)
#define	UARTIR_CR_IRFLAG_POSITIVE	(0x0U << UARTIR_CR_IRFLAG_Pos)	/*  */
#define	UARTIR_CR_IRFLAG_NEGTIVE	(0x1U << UARTIR_CR_IRFLAG_Pos)	/*  */

#define	UARTIR_CR_TH_Pos	11	/* ռձȵƲ (Tranmission High Duty) */
#define	UARTIR_CR_TH_Msk	(0xfU << UARTIR_CR_TH_Pos)

#define	UARTIR_CR_TZBRG_Pos	0	/* Ƶ (Transmission Baud Rate) */
#define	UARTIR_CR_TZBRG_Msk	(0x7ffU << UARTIR_CR_TZBRG_Pos)

#define	UARTx_CSR_BUSY_Pos	24	/* UARTͨű־ֻ
1UARTͨ
0UART */
#define	UARTx_CSR_BUSY_Msk	(0x1U << UARTx_CSR_BUSY_Pos)

#define	UARTx_CSR_TXIREN_Pos	17	/* ͺʹ
1ʹܺƷ
0رպƷ */
#define	UARTx_CSR_TXIREN_Msk	(0x1U << UARTx_CSR_TXIREN_Pos)
	/* ʹܺƷ */
	/* رպƷ */

#define	UARTx_CSR_RXTOEN_Pos	16	/* ճʱʹ
1ʹܽճʱ
0رսճʱ */
#define	UARTx_CSR_RXTOEN_Msk	(0x1U << UARTx_CSR_RXTOEN_Pos)
	/* ʹܽճʱ */
	/* رսճʱ */

#define	UARTx_CSR_IOSWAP_Pos	12	/* RXTXŽ
0Ĭ˳װͼһ£
1˳ */
#define	UARTx_CSR_IOSWAP_Msk	(0x1U << UARTx_CSR_IOSWAP_Pos)
#define	UARTx_CSR_IOSWAP_DEFAULT	(0x0U << UARTx_CSR_IOSWAP_Pos)	/* Ĭ˳װͼһ£ */
#define	UARTx_CSR_IOSWAP_EXCHANGE	(0x1U << UARTx_CSR_IOSWAP_Pos)	/* ˳ */

#define	UARTx_CSR_DMATXIFCFG_Pos	10	/* DMAжʹܣUARTͨDMAзʱЧ
1IE=1£DMAģʽ·һ֡жźһ֮֡ǰ֡ɺжź
0ǷжźIE */
#define	UARTx_CSR_DMATXIFCFG_Msk	(0x1U << UARTx_CSR_DMATXIFCFG_Pos)
	/* IE=1£DMAģʽ·һ֡жźһ֮֡ǰ֡ɺжź */
	/* ǷжźIE */

#define	UARTx_CSR_BITORD_Pos	9	/* ݷ/ʱλ˳
0LSB first
1MSB first */
#define	UARTx_CSR_BITORD_Msk	(0x1U << UARTx_CSR_BITORD_Pos)
#define	UARTx_CSR_BITORD_LSB	(0x0U << UARTx_CSR_BITORD_Pos)	/* LSB first */
#define	UARTx_CSR_BITORD_MSB	(0x1U << UARTx_CSR_BITORD_Pos)	/* MSB first */

#define	UARTx_CSR_STOPCFG_Pos	8	/* ֹͣλãԷ֡ʽЧʱжֹͣλ
01λֹͣλ
12λֹͣλ */
#define	UARTx_CSR_STOPCFG_Msk	(0x1U << UARTx_CSR_STOPCFG_Pos)
#define	UARTx_CSR_STOPCFG_1STOPBIT	(0x0U << UARTx_CSR_STOPCFG_Pos)	/* 1λֹͣλ */
#define	UARTx_CSR_STOPCFG_2STOPBIT	(0x1U << UARTx_CSR_STOPCFG_Pos)	/* 2λֹͣλ */

#define	UARTx_CSR_PDSEL_Pos	6	/* ÿ֡ݳѡ񣻴˼ĴݷͺͽͬʱЧ
007λ
018λ
109λ
116λ */
#define	UARTx_CSR_PDSEL_Msk	(0x3U << UARTx_CSR_PDSEL_Pos)
#define	UARTx_CSR_PDSEL_7BIT	(0x0U << UARTx_CSR_PDSEL_Pos)	/* 7λ */
#define	UARTx_CSR_PDSEL_8BIT	(0x1U << UARTx_CSR_PDSEL_Pos)	/* 8λ */
#define	UARTx_CSR_PDSEL_9BIT	(0x2U << UARTx_CSR_PDSEL_Pos)	/* 9λ */
#define	UARTx_CSR_PDSEL_6BIT	(0x3U << UARTx_CSR_PDSEL_Pos)	/* 6λ */

#define	UARTx_CSR_PARITY_Pos	4	/* Уλã˼ĴݷͺͽͬʱЧ
00Уλ
01żУ
10У
11RFU */
#define	UARTx_CSR_PARITY_Msk	(0x3U << UARTx_CSR_PARITY_Pos)
#define	UARTx_CSR_PARITY_NONE	(0x0U << UARTx_CSR_PARITY_Pos)	/* Уλ */
#define	UARTx_CSR_PARITY_EVEN	(0x1U << UARTx_CSR_PARITY_Pos)	/* żУ */
#define	UARTx_CSR_PARITY_ODD	(0x2U << UARTx_CSR_PARITY_Pos)	/* У */

#define	UARTx_CSR_RXPOL_Pos	3	/* ݼ
0
1ȡ */
#define	UARTx_CSR_RXPOL_Msk	(0x1U << UARTx_CSR_RXPOL_Pos)
#define	UARTx_CSR_RXPOL_POSITIVE	(0x0U << UARTx_CSR_RXPOL_Pos)	/*  */
#define	UARTx_CSR_RXPOL_NEGTIVE	(0x1U << UARTx_CSR_RXPOL_Pos)	/* ȡ */

#define	UARTx_CSR_TXPOL_Pos	2	/* ݼ
0
1ȡ */
#define	UARTx_CSR_TXPOL_Msk	(0x1U << UARTx_CSR_TXPOL_Pos)
#define	UARTx_CSR_TXPOL_POSITIVE	(0x0U << UARTx_CSR_TXPOL_Pos)	/*  */
#define	UARTx_CSR_TXPOL_NEGTIVE	(0x1U << UARTx_CSR_TXPOL_Pos)	/* ȡ */

#define	UARTx_CSR_RXEN_Pos	1	/* ʹܣ1Ч */
#define	UARTx_CSR_RXEN_Msk	(0x1U << UARTx_CSR_RXEN_Pos)
#define	UARTx_CSR_RXEN_DISBALE	(0x0U << UARTx_CSR_RXEN_Pos)
#define	UARTx_CSR_RXEN_ENABLE	(0x1U << UARTx_CSR_RXEN_Pos)

#define	UARTx_CSR_TXEN_Pos	0	/* ʹܣ1Ч */
#define	UARTx_CSR_TXEN_Msk	(0x1U << UARTx_CSR_TXEN_Pos)
#define	UARTx_CSR_TXEN_DISBALE	(0x0U << UARTx_CSR_TXEN_Pos)
#define	UARTx_CSR_TXEN_ENABLE	(0x1U << UARTx_CSR_TXEN_Pos)

#define	UARTx_IER_RXTO_IE_Pos	11	/* ճʱжʹܣ1Ч
UART0UART1Ч */
#define	UARTx_IER_RXTO_IE_Msk	(0x1U << UARTx_IER_RXTO_IE_Pos)

#define	UARTx_IER_RXERR_IE_Pos	10	/* մжʹܣ1Ч */
#define	UARTx_IER_RXERR_IE_Msk	(0x1U << UARTx_IER_RXERR_IE_Pos)

#define	UARTx_IER_RXBF_IE_Pos	8	/* ջжʹܣ1Ч */
#define	UARTx_IER_RXBF_IE_Msk	(0x1U << UARTx_IER_RXBF_IE_Pos)

#define	UARTx_IER_NEWUP_IE_Pos	7	/* RX½첽жʹܣ1ЧUART0UART1Ч */
#define	UARTx_IER_NEWUP_IE_Msk	(0x1U << UARTx_IER_NEWUP_IE_Pos)

#define	UARTx_IER_TXBE_IE_Pos	1	/* ͻжʹܣ1Ч */
#define	UARTx_IER_TXBE_IE_Msk	(0x1U << UARTx_IER_TXBE_IE_Pos)

#define	UARTx_IER_TXSE_IE_Pos	0	/* ͻҷλĴжʹܣ1Ч */
#define	UARTx_IER_TXSE_IE_Msk	(0x1U << UARTx_IER_TXSE_IE_Pos)

#define	UARTx_ISR_PERR_Pos	18	/* żУжϱ־Ӳλд1 */
#define	UARTx_ISR_PERR_Msk	(0x1U << UARTx_ISR_PERR_Pos)

#define	UARTx_ISR_FERR_Pos	17	/* ֡ʽжϱ־Ӳλд1 */
#define	UARTx_ISR_FERR_Msk	(0x1U << UARTx_ISR_FERR_Pos)

#define	UARTx_ISR_OERR_Pos	16	/* ջжϱ־ջ£յµʱλӲλд1߶ȡRXBUFʱ
ʱջԭеݱݸǡ */
#define	UARTx_ISR_OERR_Msk	(0x1U << UARTx_ISR_OERR_Pos)

#define	UARTx_ISR_RXTO_Pos	11	/* ճʱжϱ־Ӳλд1
UART0UART1Ч */
#define	UARTx_ISR_RXTO_Msk	(0x1U << UARTx_ISR_RXTO_Pos)

#define	UARTx_ISR_RXBF_Pos	8	/* ջжϱ־Ӳλд1߶ȡRXBUFʱ */
#define	UARTx_ISR_RXBF_Msk	(0x1U << UARTx_ISR_RXBF_Pos)

#define	UARTx_ISR_TXBE_Pos	1	/* ͻжϱ־ӲλдTXBUFʱ */
#define	UARTx_ISR_TXBE_Msk	(0x1U << UARTx_ISR_TXBE_Pos)

#define	UARTx_ISR_TXSE_Pos	0	/* ͻλĴжϱ־Ӳλд1дͻʱ */
#define	UARTx_ISR_TXSE_Msk	(0x1U << UARTx_ISR_TXSE_Pos)

#define	UARTx_TODR_TXDLY_LEN_Pos	8	/* ӳ٣255baud */
#define	UARTx_TODR_TXDLY_LEN_Msk	(0xffU << UARTx_TODR_TXDLY_LEN_Pos)

#define	UARTx_TODR_RXTO_LEN_Pos	0	/* ճʱȣ255baud */
#define	UARTx_TODR_RXTO_LEN_Msk	(0xffU << UARTx_TODR_RXTO_LEN_Pos)

#define	UARTx_RXBUF_RXBUF_Pos	0	/* ݻĴ */
#define	UARTx_RXBUF_RXBUF_Msk	(0x1ffU << UARTx_RXBUF_RXBUF_Pos)

#define	UARTx_TXBUF_TXBUF_Pos	0	/* ݻĴ */
#define	UARTx_TXBUF_TXBUF_Msk	(0x1ffU << UARTx_TXBUF_TXBUF_Pos)

#define	UARTx_BRG_SPBRG_Pos	0	/* ʲĴֵ */
#define	UARTx_BRG_SPBRG_Msk	(0xffffU << UARTx_BRG_SPBRG_Pos)
//Macro_End

/* Exported functions --------------------------------------------------------*/ 
extern void UART_Deinit(void);

/* ƺƷʱĬ (Infra Red Flag)
0
1 غ */
extern void UARTIR_CR_IRFLAG_Set(uint32_t SetValue);
extern uint32_t UARTIR_CR_IRFLAG_Get(void);

/* ռձȵƲ (Tranmission High Duty) غ */
extern void UARTIR_CR_TH_Set(uint32_t SetValue);
extern uint32_t UARTIR_CR_TH_Get(void);

/* Ƶ (Transmission Baud Rate) غ */
extern void UARTIR_CR_TZBRG_Set(uint32_t SetValue);
extern uint32_t UARTIR_CR_TZBRG_Get(void);
extern void UARTx_Deinit(UART_Type* UARTx);

/* UARTͨű־ֻ
1UARTͨ
0UART غ */
extern FlagStatus UARTx_CSR_BUSY_Chk(UART_Type* UARTx);

/* ͺʹ
1ʹܺƷ
0رպƷ غ */
extern void UARTx_CSR_TXIREN_Setable(UART_Type* UARTx, FunState NewState);
extern FunState UARTx_CSR_TXIREN_Getable(UART_Type* UARTx);

/* ճʱʹ
1ʹܽճʱ
0رսճʱ غ */
extern void UARTx_CSR_RXTOEN_Setable(UART_Type* UARTx, FunState NewState);
extern FunState UARTx_CSR_RXTOEN_Getable(UART_Type* UARTx);

/* RXTXŽ
0Ĭ˳װͼһ£
1˳ غ */
extern void UARTx_CSR_IOSWAP_Set(UART_Type* UARTx, uint32_t SetValue);
extern uint32_t UARTx_CSR_IOSWAP_Get(UART_Type* UARTx);

/* DMAжʹܣUARTͨDMAзʱЧ
1IE=1£DMAģʽ·һ֡жźһ֮֡ǰ֡ɺжź
0ǷжźIE غ */
extern void UARTx_CSR_DMATXIFCFG_Setable(UART_Type* UARTx, FunState NewState);
extern FunState UARTx_CSR_DMATXIFCFG_Getable(UART_Type* UARTx);

/* ݷ/ʱλ˳
0LSB first
1MSB first غ */
extern void UARTx_CSR_BITORD_Set(UART_Type* UARTx, uint32_t SetValue);
extern uint32_t UARTx_CSR_BITORD_Get(UART_Type* UARTx);

/* ֹͣλãԷ֡ʽЧʱжֹͣλ
01λֹͣλ
12λֹͣλ غ */
extern void UARTx_CSR_STOPCFG_Set(UART_Type* UARTx, uint32_t SetValue);
extern uint32_t UARTx_CSR_STOPCFG_Get(UART_Type* UARTx);

/* ÿ֡ݳѡ񣻴˼ĴݷͺͽͬʱЧ
007λ
018λ
109λ
116λ غ */
extern void UARTx_CSR_PDSEL_Set(UART_Type* UARTx, uint32_t SetValue);
extern uint32_t UARTx_CSR_PDSEL_Get(UART_Type* UARTx);

/* Уλã˼ĴݷͺͽͬʱЧ
00Уλ
01żУ
10У
11RFU غ */
extern void UARTx_CSR_PARITY_Set(UART_Type* UARTx, uint32_t SetValue);
extern uint32_t UARTx_CSR_PARITY_Get(UART_Type* UARTx);

/* ݼ
0
1ȡ غ */
extern void UARTx_CSR_RXPOL_Setable(UART_Type* UARTx,FunState NewState);
extern FunState UARTx_CSR_RXPOL_Getable(UART_Type* UARTx);

/* ݼ
0
1ȡ غ */
extern void UARTx_CSR_TXPOL_Setable(UART_Type* UARTx, FunState NewState);
extern FunState UARTx_CSR_TXPOL_Getable(UART_Type* UARTx);

/* ʹܣ1Ч غ */
extern void UARTx_CSR_RXEN_Setable(UART_Type* UARTx, FunState NewState);
extern FunState UARTx_CSR_RXEN_Getable(UART_Type* UARTx);

/* ʹܣ1Ч غ */
extern void UARTx_CSR_TXEN_Setable(UART_Type* UARTx, FunState NewState);
extern FunState UARTx_CSR_TXEN_Getable(UART_Type* UARTx);

/* ճʱжʹܣ1Ч
UART0UART1Ч غ */
extern void UARTx_IER_RXTO_IE_Setable(UART_Type* UARTx, FunState NewState);
extern FunState UARTx_IER_RXTO_IE_Getable(UART_Type* UARTx);

/* մжʹܣ1Ч غ */
extern void UARTx_IER_RXERR_IE_Setable(UART_Type* UARTx, FunState NewState);
extern FunState UARTx_IER_RXERR_IE_Getable(UART_Type* UARTx);

/* ջжʹܣ1Ч غ */
extern void UARTx_IER_RXBF_IE_Setable(UART_Type* UARTx, FunState NewState);
extern FunState UARTx_IER_RXBF_IE_Getable(UART_Type* UARTx);

/* RX½첽жʹܣ1ЧUART0UART1Ч غ */
extern void UARTx_IER_NEWUP_IE_Setable(UART_Type* UARTx, FunState NewState);
extern FunState UARTx_IER_NEWUP_IE_Getable(UART_Type* UARTx);

/* ͻжʹܣ1Ч غ */
extern void UARTx_IER_TXBE_IE_Setable(UART_Type* UARTx, FunState NewState);
extern FunState UARTx_IER_TXBE_IE_Getable(UART_Type* UARTx);

/* ͻҷλĴжʹܣ1Ч غ */
extern void UARTx_IER_TXSE_IE_Setable(UART_Type* UARTx, FunState NewState);
extern FunState UARTx_IER_TXSE_IE_Getable(UART_Type* UARTx);

/* żУжϱ־Ӳλд1 غ */
extern void UARTx_ISR_PERR_Clr(UART_Type* UARTx);
extern FlagStatus UARTx_ISR_PERR_Chk(UART_Type* UARTx);

/* ֡ʽжϱ־Ӳλд1 غ */
extern void UARTx_ISR_FERR_Clr(UART_Type* UARTx);
extern FlagStatus UARTx_ISR_FERR_Chk(UART_Type* UARTx);

/* ջжϱ־ջ£յµʱλӲλд1߶ȡRXBUFʱ
ʱջԭеݱݸǡ غ */
extern void UARTx_ISR_OERR_Clr(UART_Type* UARTx);
extern FlagStatus UARTx_ISR_OERR_Chk(UART_Type* UARTx);

/* ճʱжϱ־Ӳλд1
UART0UART1Ч غ */
extern void UARTx_ISR_RXTO_Clr(UART_Type* UARTx);
extern FlagStatus UARTx_ISR_RXTO_Chk(UART_Type* UARTx);

/* ջжϱ־Ӳλд1߶ȡRXBUFʱ غ */
extern void UARTx_ISR_RXBF_Clr(UART_Type* UARTx);
extern FlagStatus UARTx_ISR_RXBF_Chk(UART_Type* UARTx);

/* RX½첽жϱ־Ӳλд1
UART0UART1Ч غ */
extern void UARTx_ISR_NEWKF_Clr(UART_Type* UARTx);
extern FlagStatus UARTx_ISR_NEWKF_Chk(UART_Type* UARTx);

/* ͻжϱ־ӲλдTXBUFʱ غ */
extern void UARTx_ISR_TXBE_Clr(UART_Type* UARTx);
extern FlagStatus UARTx_ISR_TXBE_Chk(UART_Type* UARTx);

/* ͻλĴжϱ־Ӳλд1дͻʱ غ */
extern void UARTx_ISR_TXSE_Clr(UART_Type* UARTx);
extern FlagStatus UARTx_ISR_TXSE_Chk(UART_Type* UARTx);

/* ӳ٣255baud غ */
extern void UARTx_TODR_TXDLY_LEN_Set(UART_Type* UARTx, uint32_t SetValue);
extern uint32_t UARTx_TODR_TXDLY_LEN_Get(UART_Type* UARTx);

/* ճʱȣ255baud غ */
extern void UARTx_TODR_RXTO_LEN_Set(UART_Type* UARTx, uint32_t SetValue);
extern uint32_t UARTx_TODR_RXTO_LEN_Get(UART_Type* UARTx);

/* ݻĴ غ */
extern uint32_t UARTx_RXBUF_Read(UART_Type* UARTx);

/* ݻĴ غ */
extern void UARTx_TXBUF_Write(UART_Type* UARTx, uint32_t SetValue);
extern uint32_t UARTx_TXBUF_Read(UART_Type* UARTx);

/* ʲĴֵ غ */
extern void UARTx_BGR_Write(UART_Type* UARTx, uint32_t SetValue);
extern uint32_t UARTx_BGR_Read(UART_Type* UARTx);

extern uint32_t UART_BaudREGCalc(uint32_t BaudRate, uint32_t Clk);
extern void UART_IRModulation_Init( uint32_t ModuFreq, uint8_t ModuDutyCycle, uint32_t Clk );
extern void UART_Init(UART_Type* UARTx, UART_InitTypeDef* para);
extern void UART_SInit(UART_Type* UARTx, UART_SInitTypeDef* para,CMU_ClocksType* CMU_Clocks);

//Announce_End
#ifdef __cplusplus
}
#endif

#endif /*__FM33A0XXEV_UART_H */
