

/*----------------------------------------------------------------------------
 * Copyright (c) <2013-2015>, <Huawei Technologies Co., Ltd>
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright notice, this list of
 * conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list
 * of conditions and the following disclaimer in the documentation and/or other materials
 * provided with the distribution.
 * 3. Neither the name of the copyright holder nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific prior written
 * permission.
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *---------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------
 * Notice of Export Control Law
 * ===============================================
 * Huawei LiteOS may be subject to applicable export control laws and regulations, which might
 * include those applicable to Huawei LiteOS of U.S. and the country in which you are located.
 * Import, export and usage of Huawei LiteOS in any manner by you shall be in compliance with such
 * applicable export control laws and regulations.
 *---------------------------------------------------------------------------*/
#include "define_all.h"  

/* Kernel includes. */
#include "FreeRTOS.h"
#include "task.h"


#if (                           configUSE_TICKLESS_IDLE )

#ifdef __cplusplus

#if __cplusplus
extern "C"
{
#endif /* __cplusplus */

#endif /* __cplusplus */

void sleepConfig(void)
{
   	PMU_SleepCfg_InitTypeDef SleepCfg_InitStruct;
	CDIF_CR_INTF_EN_Setable(ENABLE);
	/*下电复位配置*/
	//pdr和bor两个下电复位至少要打开一个
	//当电源电压低于下电复位时，芯片会被复位住	
	//pdr电压档位不准但是功耗极低(几乎无测量）
	//bor电压档位准确但是需要增加2uA功耗
	RMU_PDRCR_PDREN_Setable(ENABLE);		//打开PDR
	RMU_PDRCR_PDRCFG_Set(RMU_PDRCR_PDRCFG_1P4V);//pdr电压调整到1.4V
	RMU_BORCR_OFF_BOR_Setable(ENABLE);	//关闭BOR
    VRTC_RCMFCR_EN_Setable(DISABLE);//RCMF关闭
	//VRTC_RCLPCR_RCLP_OFF_Setable(ENABLE);//RCLP关闭

	CDIF_CR_INTF_EN_Setable(DISABLE);
  
	SleepCfg_InitStruct.PMOD = PMU_CR_PMOD_SLEEP;			//功耗模式配置
	SleepCfg_InitStruct.SLPDP = PMU_CR_SLPDP_DEEPSLEEP;			//deepsleep
	SleepCfg_InitStruct.CVS = DISABLE;							//内核电压降低控制
	SleepCfg_InitStruct.SCR = 0;								//M0系统控制寄存器，一般配置为0即可	
	SleepCfg_InitStruct.TIA = PMU_WKTR_T1A_8US;//可编程额外唤醒延迟8us
	
	PMU_SleepCfg_Init(&SleepCfg_InitStruct);//休眠配置
	
	IWDT_Clr();	
	__WFI();//进入休眠
	IWDT_Clr();	
}
  /* First define the portSUPPRESS_TICKS_AND_SLEEP() macro.  The parameter is the
  time, in ticks, until the kernel next needs to execute. */
//#define portSUPPRESS_TICKS_AND_SLEEP( xIdleTime ) vApplicationSleep( xIdleTime )
#define   ulGetExternalTime()    LPTIM_CNTR_Read()
  void prvStopTickInterruptTimer ( void )
  {
    NVIC_DisableIRQ ( SysTick_IRQn );
    NVIC_ClearPendingIRQ ( SysTick_IRQn );
    SysTick->CTRL       = 0;
  }
  void prvStartTickInterruptTimer ( void )
  {
    SysTick_Config ( configCPU_CLOCK_HZ / configTICK_RATE_HZ );
    NVIC_EnableIRQ ( SysTick_IRQn );
  }

  __INLINE void disable_interrupts ( void )
  {
    __asm volatile ( "cpsid i" );
    __asm volatile ( "dsb" );
    __asm volatile ( "isb" );
  }
  __INLINE void enable_interrupts ( void )
  {
    __asm volatile ( "cpsie i" );
  }
   void vSetWakeTimeInterrupt ( TickType_t xExpectedIdleTime )
  {
    LPTIM_CCR1_Write(xExpectedIdleTime + ulGetExternalTime()); 

  }
   void prvSleep ( void )
  {
    LPTIM_IER_CC1IE_Setable(ENABLE);    

    //使能LPTIMER的外设中断
    NVIC_DisableIRQ ( LPTIM_IRQn );
    NVIC_SetPriority ( LPTIM_IRQn, 2 );
    NVIC_EnableIRQ ( LPTIM_IRQn );
    sleepConfig (); 
    LPTIM_IER_CC1IE_Setable(DISABLE);          
  }

  /* Define the function that is called by portSUPPRESS_TICKS_AND_SLEEP(). */
  void vApplicationSleep ( TickType_t xExpectedIdleTime )
  {
    unsigned long ulLowPowerTimeBeforeSleep, ulLowPowerTimeAfterSleep;
    eSleepModeStatus eSleepStatus;

    /* Read the current time from a time source that will remain operational
    while the microcontroller is in a low power state. */
    ulLowPowerTimeBeforeSleep = ulGetExternalTime ();

    /* Stop the timer that is generating the tick interrupt. */
    prvStopTickInterruptTimer ();

    /* Enter a critical section that will not effect interrupts bringing the MCU
    out of sleep mode. */
    disable_interrupts ();

    /* Ensure it is still ok to enter the sleep mode. */
    eSleepStatus        = eTaskConfirmSleepModeStatus ();

    if ( eSleepStatus == eAbortSleep )
    {
      /* A task has been moved out of the Blocked state since this macro was
      executed, or a context siwth is being held pending.  Do not enter a
      sleep state.  Restart the tick and exit the critical section. */
      prvStartTickInterruptTimer ();
      enable_interrupts ();
    }
    else 
    {
      if ( eSleepStatus == eNoTasksWaitingTimeout )
      {
        /* It is not necessary to configure an interrupt to bring the
        microcontroller out of its low power state at a fixed time in the
        future. */
        prvSleep ();
      }
      else 
      {
//        if ( xExpectedIdleTime > 0xFFFFFFFF )
//        {
//          xExpectedIdleTime   = 0xFFFFFFFF;
//        }

        /* Configure an interrupt to bring the microcontroller out of its low
        power state at the time the kernel next needs to execute.  The
        interrupt must be generated from a source that remains operational
        when the microcontroller is in a low power state. */
        vSetWakeTimeInterrupt ( xExpectedIdleTime - 1 );

        /* Enter the low power state. */
        prvSleep ();

        /* Determine how long the microcontroller was actually in a low power
        state for, which will be less than xExpectedIdleTime if the
        microcontroller was brought out of low power mode by an interrupt
        other than that configured by the vSetWakeTimeInterrupt() call.
        Note that the scheduler is suspended before
        portSUPPRESS_TICKS_AND_SLEEP() is called, and resumed when
        portSUPPRESS_TICKS_AND_SLEEP() returns.  Therefore no other tasks will
        execute until this function completes. */
        ulLowPowerTimeAfterSleep = ulGetExternalTime ();

        /* Correct the kernels tick count to account for the time the
        microcontroller spent in its low power state. */
        vTaskStepTick ( ( ulLowPowerTimeAfterSleep - ulLowPowerTimeBeforeSleep ) );
      }

      /* Exit the critical section - it might be possible to do this immediately
      after the prvSleep() calls. */
      enable_interrupts ();

      /* Restart the timer that is generating the tick interrupt. */
      prvStartTickInterruptTimer ();
    }
  }

#ifdef __cplusplus

#if __cplusplus
}


#endif /* __cplusplus */

#endif /* __cplusplus */

#endif /* end of #if (LOSCFG_KERNEL_TICKLESS == YES) */

